//
//  HPRTFunctionESC.m
//  HPRTPrinter
//
//  Created by midmirror on 16/3/18.
//  Copyright © 2016年 HPRT. All rights reserved.
//

#import "HPRTFunctionESC.h"

@implementation HPRTFunctionESC

/**
 *  打印条形码
 *
 *  @param justification    page 165. 对齐方式       取值0,1,2 : 左、中、右
 *  @param type             page 339. 取值：(A:0~6),(B:65~73)
 *  @param width            page 355. 取值：(2~6),(68~76)
 *  @param height           page 337. 取值：(1~255)
 *  @param hri_position     page 332. 取值：(0~3),(48~51)
 *  @param barcode          page 339.
 *
 *  @return 生成的条形码十六进制数据
 */
- (NSData *)escPrintBarCodeWithJustification:(int)justification
                                        Type:(int)type
                                       Width:(int)width
                                      Height:(int)height
                                 HRIPosition:(int)hri_position
                                     Barcode:(NSString *)barcode
{
    //以此为例，演示如何对 ESC-POS基础指令封装，形成功能
    HPRTCommandESC *escCommand = [[HPRTCommandESC alloc] init]; //创建对象
    
    [escCommand initCMDQueue];  //初始化存放指令的数组
    
    /**
     *  生成具体的指令，一般包括但不限定：对齐方式，打印起点，打印范围，二维码和图片宽度和高度等等。
     *  每一条指令都会加入指令数组中去。
     */
    [escCommand escSelectJustification:justification];
    [escCommand escSelectPrintPositionOfHRICharacers:hri_position];
    [escCommand escSetBarCodeWidth:width];
    [escCommand escSetBarCodeHeight:height];
    [escCommand escPrintBarCodeWithType:type Barcode:barcode];
    
    return [escCommand generateCMDData];    //从指令数组中取出命令，可以直接通过蓝牙或者 WiFi 发送出去，就可以打印一条二维码
}

/**
 *  打印二维码
 *
 *  @param justification    page 165. 对齐方式       取值0,1,2 : 左、中、右
 *  @param left_margin      page 169. 左间距
 *  @param model            page 552. 二维码样式     取值49,50 : 两种样式
 *  @param size             page 553. 大小          取值1~16
 *  @param correction_level page 555. 纠错等级       取值48~51: Level L,M,Q,H
 *  @param qr_code          page 557. 二维码数据
 *
 *  @return 生成好的十六进制数据
 */
- (NSData *)escPrintQRCodeWithJustification:(int)justification
                                 LeftMargin:(int)left_margin
                                      Model:(int)model
                                       Size:(int)size
                            CorrectionLevel:(int)correction_level
                                     QRCode:(NSString *)qr_code
{
    HPRTCommandESC *escCommand = [[HPRTCommandESC alloc] init];
    
    [escCommand initCMDQueue];
    [escCommand escSelectJustification:justification]; //1b 61
    [escCommand escSetLeftMargin:left_margin]; // 1d 4c
    [escCommand escQRCodeSelectModel:model]; //1d 28 6b 04 00 31 41
    
    [escCommand escQRCodeSetModuleSize:size]; //1d 28 6b 03 00 31 43
    [escCommand escQRCodeSelectErrorCorrectionLevel:correction_level]; //1d 28 6b 03 00 31 45
    [escCommand escQRCodeStoreDataInSymbolStorageAreaWithQRcode:qr_code];//1d 28 6b [] [] 31 50 30
    [escCommand escQRCodePrintSymbolDataInSymbolStorageArea]; //1d 28 6b 03 00 31 51 30
    [escCommand escSelectStandardMode]; //1b 53
    
    return [escCommand generateCMDData];
}

/**
 *  页模式打印
 *
 *  @param horizontal
 *  @param vertical
 *  @param width
 *  @param height
 *  @param x_position
 *  @param y_position
 *  @param direction
 *
 *  @return
 */
- (NSData *)escPrintPageWithHorizontal:(int)horizontal
                              Vertical:(int)vertical
                                 Width:(int)width
                                Height:(int)height
                             XPosition:(int)x_position
                             YPosition:(int)y_position
                             Direction:(int)direction
{
    HPRTCommandESC *escCommand = [[HPRTCommandESC alloc] init];
    
    [escCommand initCMDQueue];
    
    [escCommand escSelectPageMode]; //1b 4c
    //1b 57 []
    [escCommand escSetPrintAreaInPageModeWithHorizontal:horizontal Vertical:vertical Width:width Height:height];
    [escCommand escSetAbsolutePrintPosition:x_position]; //1b 24 []
    [escCommand escSetAbsoluteVerticalPrintPostionInPageMode:y_position]; //1d 24 []
    [escCommand escSelectPrintDirectionInPageMode:direction];//1b 54 []
    
    return [escCommand generateCMDData];
}

/**
 *  设置带格式文本
 *
 *  @param justification 对齐方式
 *  @param left_margin   左间距
 *  @param font          是否切换字体
 *  @param emphasize     加粗
 *  @param double_height 倍高
 *  @param double_width  背宽
 *  @param underline     下划线
 */
- (NSData *)escPrintTextWithJustification:(int)justification
                                   Margin:(int)left_margin
                                     Font:(BOOL)font
                                Emphasize:(BOOL)emphasize
                             DoubleHeight:(BOOL)double_height
                              DoubleWidth:(BOOL)double_width
                                Underline:(BOOL)underline
                              ReverseMode:(BOOL)reverse_mode
{
    HPRTCommandESC *escCommand = [[HPRTCommandESC alloc] init];
    
    [escCommand initCMDQueue];
    
    [escCommand escSelectJustification:justification];
    [escCommand escSetLeftMargin:left_margin];
    
    
    int mode = 0;
    int reverse = 0;
    
    if (font)          mode += 1;
    if (emphasize)     mode += 8;
    if (double_height) mode += 16;
    if (double_width)  mode += 32;
    if (underline)     mode += 128;
    
    if (reverse_mode)  reverse = 1;
    
    [escCommand escSelectPrintMode:mode];
    
    [escCommand escTurnWhiteBlackReversePrintMode:reverse];
    
    return [escCommand generateCMDData];
}

@end
