//
//  HPRTDispatcher.h
//
//  Created by midmirror on 15/12/25.
//  Copyright © 2015年 midmirror. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreBluetooth/CoreBluetooth.h>
#import "HPRTPublicDefine.h"

@class HPRTDispatcher;

/**
 *  Protocol 功能：一些蓝牙和 WiFi 建立连接时的代理方法。注册相应的通知也可以实现一样的功能
 */
@protocol HPRTDispatchProtocol <NSObject>

@optional

/**
 *  每发现一个蓝牙设备调用一次 或者观察通知：PRTBluetoothUpdatePeripheralList（携带数据）
 */
- (void)didDiscoverPeripheral:(CBPeripheral *)peripheral RSSI:(NSNumber *)RSSI;

/**
 *  成功连接 BLE 或者观察通知：HPRTDispatchModeChangeToBLE
 */
- (void)didConnectPeripheral:(CBPeripheral *)peripheral;

/**
 *  成功连接 WiFi 或者观察通知：HPRTDispatchModeChangeToWIFI
 */
- (void)didConnectWiFi:(NSString *)ip;

@end

/**
 *  主要功能：实现 SDK 和打印机的通讯，主要是BLE 蓝牙和 WiFi 两种方式通讯
 */
@interface HPRTDispatcher : NSObject

singletonH(HPRTDispatcher)

/*
 *  @enum PRTTransmitDispatchMode
 *
 *  @discussion Represents the current transmit dispatch mode of a PRTTransmitDispatchMode.
 *
 *  @constant PRTPrinterModeOffline     Offline Mode    离线模式
 *  @constant PRTPrinterModeBle         BLE     Mode    蓝牙连接模式
 *  @constant PRTPrinterModeWifi        Wifi    Mode    Wifi连接模式
 *
 */
typedef NS_ENUM(NSInteger, DispatchMode) {
    
    DispatchModeOffline = 0,
    DispatchModeBLE,
    DispatchModeWIFI,
};

typedef NS_ENUM(NSInteger, InstructionMode) {
    
    InstructionModeESC_POS = 0,
    InstructionModeTSC_TSPL,
};

@property(assign,nonatomic,readwrite) NSInteger currentInstructionMode;
@property(assign,nonatomic,readwrite) NSInteger currentDispatchMode;
@property(strong,nonatomic,readwrite) CBPeripheral *bleConnected;
@property(strong,nonatomic,readwrite) NSString *wifiConnectedName;

/**
 *  send to Printer with NSData
 */
- (void)sendData:(NSData *)data;

/**
 *  You just need to create several NSData object in a NSArray, then it will send to print one by one.
 *  将需要的一组数据(NSData)加入 NSArray,将会依次打印出来。
 *
 *  @param dataQueue like @[data1,data2,data3]
 */
- (void)sendDataQueue:(NSArray *)dataQueue;


#pragma mark CurrentConnect

/**
 *  取消当前连接（Wifi 或者 BLE）
 */
- (void)disConnectCurrentLine;

#pragma mark HPRTBluetooth

/**
 *  扫描蓝牙
 */
- (void)scanBluetooth;

/**
 *  停止扫描蓝牙
 */
- (void)stopScanBluetooth;

/**
 *  连接蓝牙
 */
- (void)connectBluetooth:(CBPeripheral *)peripheral;

/**
 *  断开连接蓝牙
 */
- (void)disConnectBluetooth;

/**
 *  设置 BLE 代理
 */
- (void)setBLEDelegate:(id)someone;

/**
 *  设置 WiFi 代理
 */
- (void)setWiFiDelegate:(id)someone;

/**
 *  根据 rssi 强度计算距离
 *
 *  @param rssi 信号强度
 *  @param a    发射端和接收端相隔1米时的信号强度
 *  @param n    环境衰减因子
 *
 */
- (NSString *)generateDistanceWithRSSI:(int)rssi;

#pragma mark HPRTNetWork

/**
 *  获取当前 Wifi 的路由信息
 *
 *  @return 路由信息字典
 */
- (NSDictionary *)getRouterInfo;

/**
 *  通过 ip 地址连接 wifi
 *
 */
- (void)connectWifi:(NSString *)ip;

/**
 *  判断是否是 ESC—POS 指令集
 */
- (BOOL)isEscPosInstruction:(NSString *)name;

/**
 *  判断是否是 TSC-TSPL 指令集
 */
- (BOOL)isTscTsplInstrution:(NSString *)name;

@end
